@@ -1,3 +1,4 @@
+use crate::docstring::Docstring;
 use crate::goto::{GotoTarget, find_goto_target};
 use crate::{Db, MarkupKind, RangedValue};
 use ruff_db::files::{File, FileRange};
@@ -9,7 +10,7 @@ use ty_python_semantic::SemanticModel;
 use ty_python_semantic::types::Type;
 
 pub fn hover(db: &dyn Db, file: File, offset: TextSize) -> Option<RangedValue<Hover<'_>>> {
-    let parsed = parsed_module(db.upcast(), file).load(db.upcast());
+    let parsed = parsed_module(db, file).load(db);
     let goto_target = find_goto_target(&parsed, offset)?;
 
     if let GotoTarget::Expression(expr) = goto_target {
@@ -18,17 +19,29 @@ pub fn hover(db: &dyn Db, file: File, offset: TextSize) -> Option<RangedValue<Ho
         }
     }
 
-    let model = SemanticModel::new(db.upcast(), file);
-    let ty = goto_target.inferred_type(&model)?;
+    let model = SemanticModel::new(db, file);
+    let ty = goto_target.inferred_type(&model).map(HoverContent::Type);
+    let docs = goto_target
+        .get_definition_targets(
+            file,
+            db,
+            ty_python_semantic::ImportAliasResolution::ResolveAliases,
+        )
+        .and_then(|definitions| definitions.docstring(db))
+        .map(HoverContent::Docstring);
 
-    tracing::debug!(
-        "Inferred type of covering node is {}",
-        ty.display(db.upcast())
-    );
+    if let Some(HoverContent::Type(ty)) = ty {
+        tracing::debug!("Inferred type of covering node is {}", ty.display(db));
+    }
 
-    // TODO: Add documentation of the symbol (not the type's definition).
     // TODO: Render the symbol's signature instead of just its type.
-    let contents = vec![HoverContent::Type(ty)];
+    let mut contents = Vec::new();
+    contents.extend(ty);
+    contents.extend(docs);
+
+    if contents.is_empty() {
+        return None;
+    }
 
     Some(RangedValue {
         range: FileRange::new(file, goto_target.range()),
@@ -98,6 +111,7 @@ impl fmt::Display for DisplayHover<'_> {
 #[derive(Debug, Clone, Eq, PartialEq)]
 pub enum HoverContent<'db> {
     Type(Type<'db>),
+    Docstring(Docstring),
 }
 
 impl<'db> HoverContent<'db> {
@@ -121,8 +135,9 @@ impl fmt::Display for DisplayHoverContent<'_, '_> {
         match self.content {
             HoverContent::Type(ty) => self
                 .kind
-                .fenced_code_block(ty.display(self.db.upcast()), "text")
+                .fenced_code_block(ty.display(self.db), "python")
                 .fmt(f),
+            HoverContent::Docstring(docstring) => docstring.render(self.kind).fmt(f),
         }
     }
 }
@@ -132,7 +147,6 @@ mod tests {
     use crate::tests::{CursorTest, cursor_test};
     use crate::{MarkupKind, hover};
     use insta::assert_snapshot;
-    use ruff_db::Upcast;
     use ruff_db::diagnostic::{
         Annotation, Diagnostic, DiagnosticFormat, DiagnosticId, DisplayDiagnosticConfig, LintName,
         Severity, Span,
@@ -152,7 +166,7 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         Literal[10]
         ---------------------------------------------
-        ```text
+        ```python
         Literal[10]
         ```
         ---------------------------------------------
@@ -169,6 +183,261 @@ mod tests {
         ");
     }
 
+    #[test]
+    fn hover_function() {
+        let test = cursor_test(
+            r#"
+        def my_func(a, b):
+            '''This is such a great func!!
+
+            Args:
+                a: first for a reason
+                b: coming for `a`'s title
+            '''
+            return 0
+
+        my_fu<CURSOR>nc(1, 2)
+        "#,
+        );
+
+        assert_snapshot!(test.hover(), @r"
+        def my_func(a, b) -> Unknown
+        ---------------------------------------------
+        This is such a great func!!
+
+        Args:
+            a: first for a reason
+            b: coming for `a`'s title
+
+        ---------------------------------------------
+        ```python
+        def my_func(a, b) -> Unknown
+        ```
+        ---
+        ```text
+        This is such a great func!!
+
+        Args:
+            a: first for a reason
+            b: coming for `a`'s title
+
+        ```
+        ---------------------------------------------
+        info[hover]: Hovered content is
+          --> main.py:11:9
+           |
+         9 |             return 0
+        10 |
+        11 |         my_func(1, 2)
+           |         ^^^^^-^
+           |         |    |
+           |         |    Cursor offset
+           |         source
+           |
+        ");
+    }
+
+    #[test]
+    fn hover_class() {
+        let test = cursor_test(
+            r#"
+        class MyClass:
+            '''
+                This is such a great class!!
+
+                    Don't you know?
+                
+                Everyone loves my class!!
+
+            '''
+            def __init__(self, val):
+                """initializes MyClass (perfectly)"""
+                self.val = val
+            
+            def my_method(self, a, b):
+                '''This is such a great func!!
+
+                Args:
+                    a: first for a reason
+                    b: coming for `a`'s title
+                '''
+                return 0
+
+        MyCla<CURSOR>ss
+        "#,
+        );
+
+        assert_snapshot!(test.hover(), @r"
+        <class 'MyClass'>
+        ---------------------------------------------
+        This is such a great class!!
+
+            Don't you know?
+
+        Everyone loves my class!!
+
+        ---------------------------------------------
+        ```python
+        <class 'MyClass'>
+        ```
+        ---
+        ```text
+        This is such a great class!!
+
+            Don't you know?
+
+        Everyone loves my class!!
+
+        ```
+        ---------------------------------------------
+        info[hover]: Hovered content is
+          --> main.py:24:9
+           |
+        22 |                 return 0
+        23 |
+        24 |         MyClass
+           |         ^^^^^-^
+           |         |    |
+           |         |    Cursor offset
+           |         source
+           |
+        ");
+    }
+
+    #[test]
+    fn hover_class_init() {
+        let test = cursor_test(
+            r#"
+        class MyClass:
+            '''
+                This is such a great class!!
+
+                    Don't you know?
+                
+                Everyone loves my class!!
+
+            '''
+            def __init__(self, val):
+                """initializes MyClass (perfectly)"""
+                self.val = val
+            
+            def my_method(self, a, b):
+                '''This is such a great func!!
+
+                Args:
+                    a: first for a reason
+                    b: coming for `a`'s title
+                '''
+                return 0
+
+        x = MyCla<CURSOR>ss(0)
+        "#,
+        );
+
+        assert_snapshot!(test.hover(), @r"
+        <class 'MyClass'>
+        ---------------------------------------------
+        This is such a great class!!
+
+            Don't you know?
+
+        Everyone loves my class!!
+
+        ---------------------------------------------
+        ```python
+        <class 'MyClass'>
+        ```
+        ---
+        ```text
+        This is such a great class!!
+
+            Don't you know?
+
+        Everyone loves my class!!
+
+        ```
+        ---------------------------------------------
+        info[hover]: Hovered content is
+          --> main.py:24:13
+           |
+        22 |                 return 0
+        23 |
+        24 |         x = MyClass(0)
+           |             ^^^^^-^
+           |             |    |
+           |             |    Cursor offset
+           |             source
+           |
+        ");
+    }
+
+    #[test]
+    fn hover_class_method() {
+        let test = cursor_test(
+            r#"
+        class MyClass:
+            '''
+                This is such a great class!!
+
+                    Don't you know?
+                
+                Everyone loves my class!!
+
+            '''
+            def __init__(self, val):
+                """initializes MyClass (perfectly)"""
+                self.val = val
+            
+            def my_method(self, a, b):
+                '''This is such a great func!!
+
+                Args:
+                    a: first for a reason
+                    b: coming for `a`'s title
+                '''
+                return 0
+
+        x = MyClass(0)
+        x.my_me<CURSOR>thod(2, 3)
+        "#,
+        );
+
+        assert_snapshot!(test.hover(), @r"
+        bound method MyClass.my_method(a, b) -> Unknown
+        ---------------------------------------------
+        This is such a great func!!
+
+        Args:
+            a: first for a reason
+            b: coming for `a`'s title
+
+        ---------------------------------------------
+        ```python
+        bound method MyClass.my_method(a, b) -> Unknown
+        ```
+        ---
+        ```text
+        This is such a great func!!
+
+        Args:
+            a: first for a reason
+            b: coming for `a`'s title
+
+        ```
+        ---------------------------------------------
+        info[hover]: Hovered content is
+          --> main.py:25:11
+           |
+        24 |         x = MyClass(0)
+        25 |         x.my_method(2, 3)
+           |           ^^^^^-^^^
+           |           |    |
+           |           |    Cursor offset
+           |           source
+           |
+        ");
+    }
+
     #[test]
     fn hover_member() {
         let test = cursor_test(
@@ -188,19 +457,19 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         int
         ---------------------------------------------
-        ```text
+        ```python
         int
         ```
         ---------------------------------------------
         info[hover]: Hovered content is
-          --> main.py:10:9
+          --> main.py:10:13
            |
          9 |         foo = Foo()
         10 |         foo.a
-           |         ^^^^-
-           |         |   |
-           |         |   Cursor offset
-           |         source
+           |             -
+           |             |
+           |             source
+           |             Cursor offset
            |
         ");
     }
@@ -218,7 +487,7 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         def foo(a, b) -> Unknown
         ---------------------------------------------
-        ```text
+        ```python
         def foo(a, b) -> Unknown
         ```
         ---------------------------------------------
@@ -247,7 +516,7 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         bool
         ---------------------------------------------
-        ```text
+        ```python
         bool
         ```
         ---------------------------------------------
@@ -268,9 +537,15 @@ mod tests {
     fn hover_keyword_parameter() {
         let test = cursor_test(
             r#"
-            def test(a: int): ...
+            def test(ab: int):
+                """my cool test
 
-            test(a<CURSOR>= 123)
+                Args:
+                    ab: a nice little integer
+                """
+                return 0
+
+            test(a<CURSOR>b= 123)
             "#,
         );
 
@@ -278,20 +553,21 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         Literal[123]
         ---------------------------------------------
-        ```text
+        ```python
         Literal[123]
         ```
         ---------------------------------------------
         info[hover]: Hovered content is
-         --> main.py:4:18
-          |
-        2 |             def test(a: int): ...
-        3 |
-        4 |             test(a= 123)
-          |                  ^- Cursor offset
-          |                  |
-          |                  source
-          |
+          --> main.py:10:18
+           |
+         8 |                 return 0
+         9 |
+        10 |             test(ab= 123)
+           |                  ^-
+           |                  ||
+           |                  |Cursor offset
+           |                  source
+           |
         ");
     }
 
@@ -300,9 +576,13 @@ mod tests {
         let test = cursor_test(
             r#"
 
-            def foo(a, b): ...
+            def foo(a, b):
+                """The foo function"""
+                return 0
 
-            def bar(a, b): ...
+            def bar(a, b):
+                """The bar function"""
+                return 1
 
             if random.choice([True, False]):
                 a = foo
@@ -316,16 +596,16 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         (def foo(a, b) -> Unknown) | (def bar(a, b) -> Unknown)
         ---------------------------------------------
-        ```text
+        ```python
         (def foo(a, b) -> Unknown) | (def bar(a, b) -> Unknown)
         ```
         ---------------------------------------------
         info[hover]: Hovered content is
-          --> main.py:12:13
+          --> main.py:16:13
            |
-        10 |                 a = bar
-        11 |
-        12 |             a
+        14 |                 a = bar
+        15 |
+        16 |             a
            |             ^- Cursor offset
            |             |
            |             source
@@ -343,13 +623,36 @@ mod tests {
             "#,
         );
 
-        test.write_file("lib.py", "a = 10").unwrap();
+        test.write_file(
+            "lib.py",
+            r"
+        '''
+        The cool lib_py module!
+
+        Wow this module rocks.
+        '''
+        a = 10
+        ",
+        )
+        .unwrap();
 
         assert_snapshot!(test.hover(), @r"
         <module 'lib'>
         ---------------------------------------------
-        ```text
+        The cool lib_py module!
+
+        Wow this module rocks.
+
+        ---------------------------------------------
+        ```python
         <module 'lib'>
+        ```
+        ---
+        ```text
+        The cool lib_py module!
+
+        Wow this module rocks.
+
         ```
         ---------------------------------------------
         info[hover]: Hovered content is
@@ -366,6 +669,56 @@ mod tests {
         ");
     }
 
+    #[test]
+    fn hover_module_import() {
+        let mut test = cursor_test(
+            r#"
+            import li<CURSOR>b
+
+            lib
+            "#,
+        );
+
+        test.write_file(
+            "lib.py",
+            r"
+        '''
+        The cool lib_py module!
+
+        Wow this module rocks.
+        '''
+        a = 10
+        ",
+        )
+        .unwrap();
+
+        assert_snapshot!(test.hover(), @r"
+        The cool lib_py module!
+
+        Wow this module rocks.
+
+        ---------------------------------------------
+        ```text
+        The cool lib_py module!
+
+        Wow this module rocks.
+
+        ```
+        ---------------------------------------------
+        info[hover]: Hovered content is
+         --> main.py:2:20
+          |
+        2 |             import lib
+          |                    ^^-
+          |                    | |
+          |                    | Cursor offset
+          |                    source
+        3 |
+        4 |             lib
+          |
+        ");
+    }
+
     #[test]
     fn hover_type_of_expression_with_type_var_type() {
         let test = cursor_test(
@@ -374,11 +727,12 @@ mod tests {
             "#,
         );
 
-        assert_snapshot!(test.hover(), @r"
-        T
+        // TODO: This should render T@Alias once we create GenericContexts for type alias scopes.
+        assert_snapshot!(test.hover(), @r###"
+        typing.TypeVar
         ---------------------------------------------
-        ```text
-        T
+        ```python
+        typing.TypeVar
         ```
         ---------------------------------------------
         info[hover]: Hovered content is
@@ -389,7 +743,7 @@ mod tests {
           |                                              |
           |                                              source
           |
-        ");
+        "###);
     }
 
     #[test]
@@ -403,7 +757,7 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         @Todo
         ---------------------------------------------
-        ```text
+        ```python
         @Todo
         ```
         ---------------------------------------------
@@ -429,7 +783,7 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         @Todo
         ---------------------------------------------
-        ```text
+        ```python
         @Todo
         ```
         ---------------------------------------------
@@ -455,7 +809,7 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         Literal[1]
         ---------------------------------------------
-        ```text
+        ```python
         Literal[1]
         ```
         ---------------------------------------------
@@ -486,7 +840,7 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         Literal[1]
         ---------------------------------------------
-        ```text
+        ```python
         Literal[1]
         ```
         ---------------------------------------------
@@ -516,19 +870,19 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         Literal[2]
         ---------------------------------------------
-        ```text
+        ```python
         Literal[2]
         ```
         ---------------------------------------------
         info[hover]: Hovered content is
-         --> main.py:5:13
+         --> main.py:5:15
           |
         3 |                 attr: int = 1
         4 |
         5 |             C.attr = 2
-          |             ^^^^^^- Cursor offset
-          |             |
-          |             source
+          |               ^^^^- Cursor offset
+          |               |
+          |               source
           |
         ");
     }
@@ -549,19 +903,19 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         Unknown | Literal[1]
         ---------------------------------------------
-        ```text
+        ```python
         Unknown | Literal[1]
         ```
         ---------------------------------------------
         info[hover]: Hovered content is
-         --> main.py:5:13
+         --> main.py:5:15
           |
         3 |                 attr = 1
         4 |
         5 |             C.attr += 2
-          |             ^^^^^^- Cursor offset
-          |             |
-          |             source
+          |               ^^^^- Cursor offset
+          |               |
+          |               source
           |
         ");
     }
@@ -578,7 +932,7 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         int
         ---------------------------------------------
-        ```text
+        ```python
         int
         ```
         ---------------------------------------------
@@ -606,7 +960,7 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         Literal[1]
         ---------------------------------------------
-        ```text
+        ```python
         Literal[1]
         ```
         ---------------------------------------------
@@ -635,19 +989,19 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         int
         ---------------------------------------------
-        ```text
+        ```python
         int
         ```
         ---------------------------------------------
         info[hover]: Hovered content is
-         --> main.py:4:17
+         --> main.py:4:22
           |
         2 |         class Foo:
         3 |             def __init__(self, a: int):
         4 |                 self.a: int = a
-          |                 ^^^^^^- Cursor offset
-          |                 |
-          |                 source
+          |                      ^- Cursor offset
+          |                      |
+          |                      source
           |
         ");
     }
@@ -657,6 +1011,12 @@ mod tests {
         let test = cursor_test(
             r#"
             def foo(a: str | None, b):
+                '''
+                    My cool func
+                
+                    Args:
+                        a: hopefully a string, right?!   
+                '''
                 if a is not None:
                     print(a<CURSOR>)
             "#,
@@ -665,20 +1025,20 @@ mod tests {
         assert_snapshot!(test.hover(), @r"
         str
         ---------------------------------------------
-        ```text
+        ```python
         str
         ```
         ---------------------------------------------
         info[hover]: Hovered content is
-         --> main.py:4:27
-          |
-        2 |             def foo(a: str | None, b):
-        3 |                 if a is not None:
-        4 |                     print(a)
-          |                           ^- Cursor offset
-          |                           |
-          |                           source
-          |
+          --> main.py:10:27
+           |
+         8 |                 '''
+         9 |                 if a is not None:
+        10 |                     print(a)
+           |                           ^- Cursor offset
+           |                           |
+           |                           source
+           |
         ");
     }
 
@@ -774,7 +1134,7 @@ mod tests {
                 .message("Cursor offset"),
             );
 
-            write!(buf, "{}", diagnostic.display(&self.db.upcast(), &config)).unwrap();
+            write!(buf, "{}", diagnostic.display(&self.db, &config)).unwrap();
 
             buf
         }
