@@ -1,10 +1,11 @@
 use std::fmt::Formatter;
 use std::str::FromStr;
-use std::sync::Arc;
 
 use ruff_db::files::File;
 use ruff_python_ast::name::Name;
 use ruff_python_stdlib::identifiers::is_identifier;
+use salsa::Database;
+use salsa::plumbing::AsId;
 
 use super::path::SearchPath;
 use crate::Db;
@@ -12,13 +13,19 @@ use crate::module_name::ModuleName;
 use crate::module_resolver::path::SystemOrVendoredPathRef;
 
 /// Representation of a Python module.
-#[derive(Clone, PartialEq, Eq, Hash, get_size2::GetSize)]
-pub struct Module {
-    inner: Arc<ModuleInner>,
+#[derive(Clone, Copy, Eq, Hash, PartialEq, salsa::Supertype, salsa::Update)]
+pub enum Module<'db> {
+    File(FileModule<'db>),
+    Namespace(NamespacePackage<'db>),
 }
 
-impl Module {
+// The Salsa heap is tracked separately.
+impl get_size2::GetSize for Module<'_> {}
+
+#[salsa::tracked]
+impl<'db> Module<'db> {
     pub(crate) fn file_module(
+        db: &'db dyn Db,
         name: ModuleName,
         kind: ModuleKind,
         search_path: SearchPath,
@@ -26,67 +33,57 @@ impl Module {
     ) -> Self {
         let known = KnownModule::try_from_search_path_and_name(&search_path, &name);
 
-        Self {
-            inner: Arc::new(ModuleInner::FileModule {
-                name,
-                kind,
-                search_path,
-                file,
-                known,
-            }),
-        }
+        Self::File(FileModule::new(db, name, kind, search_path, file, known))
     }
 
-    pub(crate) fn namespace_package(name: ModuleName) -> Self {
-        Self {
-            inner: Arc::new(ModuleInner::NamespacePackage { name }),
-        }
+    pub(crate) fn namespace_package(db: &'db dyn Db, name: ModuleName) -> Self {
+        Self::Namespace(NamespacePackage::new(db, name))
     }
 
     /// The absolute name of the module (e.g. `foo.bar`)
-    pub fn name(&self) -> &ModuleName {
-        match &*self.inner {
-            ModuleInner::FileModule { name, .. } => name,
-            ModuleInner::NamespacePackage { name, .. } => name,
+    pub fn name(self, db: &'db dyn Database) -> &'db ModuleName {
+        match self {
+            Module::File(module) => module.name(db),
+            Module::Namespace(ref package) => package.name(db),
         }
     }
 
     /// The file to the source code that defines this module
     ///
     /// This is `None` for namespace packages.
-    pub fn file(&self) -> Option<File> {
-        match &*self.inner {
-            ModuleInner::FileModule { file, .. } => Some(*file),
-            ModuleInner::NamespacePackage { .. } => None,
+    pub fn file(self, db: &'db dyn Database) -> Option<File> {
+        match self {
+            Module::File(module) => Some(module.file(db)),
+            Module::Namespace(_) => None,
         }
     }
 
     /// Is this a module that we special-case somehow? If so, which one?
-    pub fn known(&self) -> Option<KnownModule> {
-        match &*self.inner {
-            ModuleInner::FileModule { known, .. } => *known,
-            ModuleInner::NamespacePackage { .. } => None,
+    pub(crate) fn known(self, db: &'db dyn Database) -> Option<KnownModule> {
+        match self {
+            Module::File(module) => module.known(db),
+            Module::Namespace(_) => None,
         }
     }
 
     /// Does this module represent the given known module?
-    pub fn is_known(&self, known_module: KnownModule) -> bool {
-        self.known() == Some(known_module)
+    pub(crate) fn is_known(self, db: &'db dyn Database, known_module: KnownModule) -> bool {
+        self.known(db) == Some(known_module)
     }
 
     /// The search path from which the module was resolved.
-    pub(crate) fn search_path(&self) -> Option<&SearchPath> {
-        match &*self.inner {
-            ModuleInner::FileModule { search_path, .. } => Some(search_path),
-            ModuleInner::NamespacePackage { .. } => None,
+    pub(crate) fn search_path(self, db: &'db dyn Database) -> Option<&'db SearchPath> {
+        match self {
+            Module::File(module) => Some(module.search_path(db)),
+            Module::Namespace(_) => None,
         }
     }
 
     /// Determine whether this module is a single-file module or a package
-    pub fn kind(&self) -> ModuleKind {
-        match &*self.inner {
-            ModuleInner::FileModule { kind, .. } => *kind,
-            ModuleInner::NamespacePackage { .. } => ModuleKind::Package,
+    pub fn kind(self, db: &'db dyn Database) -> ModuleKind {
+        match self {
+            Module::File(module) => module.kind(db),
+            Module::Namespace(_) => ModuleKind::Package,
         }
     }
 
@@ -97,47 +94,75 @@ impl Module {
     ///
     /// The names returned correspond to the "base" name of the module.
     /// That is, `{self.name}.{basename}` should give the full module name.
-    pub fn all_submodules(&self, db: &dyn Db) -> Vec<Name> {
+    pub fn all_submodules(self, db: &'db dyn Db) -> &'db [Name] {
         self.all_submodules_inner(db).unwrap_or_default()
     }
 
-    fn all_submodules_inner(&self, db: &dyn Db) -> Option<Vec<Name>> {
-        fn is_submodule(
-            is_dir: bool,
-            is_file: bool,
-            basename: Option<&str>,
-            extension: Option<&str>,
-        ) -> bool {
-            is_dir
-                || (is_file
-                    && matches!(extension, Some("py" | "pyi"))
-                    && !matches!(basename, Some("__init__.py" | "__init__.pyi")))
-        }
-
+    fn all_submodules_inner(self, db: &'db dyn Db) -> Option<&'db [Name]> {
         // It would be complex and expensive to compute all submodules for
         // namespace packages, since a namespace package doesn't correspond
         // to a single file; it can span multiple directories across multiple
         // search paths. For now, we only compute submodules for traditional
         // packages that exist in a single directory on a single search path.
-        let ModuleInner::FileModule {
-            kind: ModuleKind::Package,
-            file,
-            ..
-        } = &*self.inner
-        else {
+        let Module::File(module) = self else {
             return None;
         };
+        if !matches!(module.kind(db), ModuleKind::Package) {
+            return None;
+        }
+        all_submodule_names_for_package(db, module.file(db)).as_deref()
+    }
+}
 
-        let path = SystemOrVendoredPathRef::try_from_file(db, *file)?;
-        debug_assert!(
-            matches!(path.file_name(), Some("__init__.py" | "__init__.pyi")),
-            "expected package file `{:?}` to be `__init__.py` or `__init__.pyi`",
-            path.file_name(),
-        );
+impl std::fmt::Debug for Module<'_> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        salsa::with_attached_database(|db| {
+            f.debug_struct("Module")
+                .field("name", &self.name(db))
+                .field("kind", &self.kind(db))
+                .field("file", &self.file(db))
+                .field("search_path", &self.search_path(db))
+                .field("known", &self.known(db))
+                .finish()
+        })
+        .unwrap_or_else(|| f.debug_tuple("Module").field(&self.as_id()).finish())
+    }
+}
 
-        Some(match path.parent()? {
-            SystemOrVendoredPathRef::System(parent_directory) => db
-                .system()
+#[allow(clippy::ref_option)]
+#[salsa::tracked(returns(ref))]
+fn all_submodule_names_for_package(db: &dyn Db, file: File) -> Option<Vec<Name>> {
+    fn is_submodule(
+        is_dir: bool,
+        is_file: bool,
+        basename: Option<&str>,
+        extension: Option<&str>,
+    ) -> bool {
+        is_dir
+            || (is_file
+                && matches!(extension, Some("py" | "pyi"))
+                && !matches!(basename, Some("__init__.py" | "__init__.pyi")))
+    }
+
+    let path = SystemOrVendoredPathRef::try_from_file(db, file)?;
+    debug_assert!(
+        matches!(path.file_name(), Some("__init__.py" | "__init__.pyi")),
+        "expected package file `{:?}` to be `__init__.py` or `__init__.pyi`",
+        path.file_name(),
+    );
+
+    Some(match path.parent()? {
+        SystemOrVendoredPathRef::System(parent_directory) => {
+            // Read the revision on the corresponding file root to
+            // register an explicit dependency on this directory
+            // tree. When the revision gets bumped, the cache
+            // that Salsa creates does for this routine will be
+            // invalidated.
+            if let Some(root) = db.files().root(db, parent_directory) {
+                let _ = root.revision(db);
+            }
+
+            db.system()
                 .read_directory(parent_directory)
                 .inspect_err(|err| {
                     tracing::debug!(
@@ -161,57 +186,50 @@ impl Module {
                     let stem = entry.path().file_stem()?;
                     is_identifier(stem).then(|| Name::from(stem))
                 })
-                .collect(),
-            SystemOrVendoredPathRef::Vendored(parent_directory) => db
-                .vendored()
-                .read_directory(parent_directory)
-                .into_iter()
-                .filter(|entry| {
-                    let ty = entry.file_type();
-                    let path = entry.path();
-                    is_submodule(
-                        ty.is_directory(),
-                        ty.is_file(),
-                        path.file_name(),
-                        path.extension(),
-                    )
-                })
-                .filter_map(|entry| {
-                    let stem = entry.path().file_stem()?;
-                    is_identifier(stem).then(|| Name::from(stem))
-                })
-                .collect(),
-        })
-    }
+                .collect()
+        }
+        SystemOrVendoredPathRef::Vendored(parent_directory) => db
+            .vendored()
+            .read_directory(parent_directory)
+            .into_iter()
+            .filter(|entry| {
+                let ty = entry.file_type();
+                let path = entry.path();
+                is_submodule(
+                    ty.is_directory(),
+                    ty.is_file(),
+                    path.file_name(),
+                    path.extension(),
+                )
+            })
+            .filter_map(|entry| {
+                let stem = entry.path().file_stem()?;
+                is_identifier(stem).then(|| Name::from(stem))
+            })
+            .collect(),
+    })
 }
 
-impl std::fmt::Debug for Module {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        f.debug_struct("Module")
-            .field("name", &self.name())
-            .field("kind", &self.kind())
-            .field("file", &self.file())
-            .field("search_path", &self.search_path())
-            .field("known", &self.known())
-            .finish()
-    }
+/// A module that resolves to a file (`lib.py` or `package/__init__.py`)
+#[salsa::tracked(debug, heap_size=ruff_memory_usage::heap_size)]
+pub struct FileModule<'db> {
+    #[returns(ref)]
+    name: ModuleName,
+    kind: ModuleKind,
+    #[returns(ref)]
+    search_path: SearchPath,
+    file: File,
+    known: Option<KnownModule>,
 }
 
-#[derive(PartialEq, Eq, Hash, get_size2::GetSize)]
-enum ModuleInner {
-    /// A module that resolves to a file (`lib.py` or `package/__init__.py`)
-    FileModule {
-        name: ModuleName,
-        kind: ModuleKind,
-        search_path: SearchPath,
-        file: File,
-        known: Option<KnownModule>,
-    },
-
-    /// A namespace package. Namespace packages are special because
-    /// there are multiple possible paths and they have no corresponding
-    /// code file.
-    NamespacePackage { name: ModuleName },
+/// A namespace package.
+///
+/// Namespace packages are special because there are
+/// multiple possible paths and they have no corresponding code file.
+#[salsa::tracked(debug, heap_size=ruff_memory_usage::heap_size)]
+pub struct NamespacePackage<'db> {
+    #[returns(ref)]
+    name: ModuleName,
 }
 
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Hash, get_size2::GetSize)]
@@ -259,10 +277,11 @@ pub enum KnownModule {
     UnittestMock,
     #[cfg(test)]
     Uuid,
+    Warnings,
 }
 
 impl KnownModule {
-    pub const fn as_str(self) -> &'static str {
+    pub(crate) const fn as_str(self) -> &'static str {
         match self {
             Self::Builtins => "builtins",
             Self::Enum => "enum",
@@ -278,6 +297,7 @@ impl KnownModule {
             Self::TypeCheckerInternals => "_typeshed._type_checker_internals",
             Self::TyExtensions => "ty_extensions",
             Self::ImportLib => "importlib",
+            Self::Warnings => "warnings",
             #[cfg(test)]
             Self::UnittestMock => "unittest.mock",
             #[cfg(test)]
@@ -285,7 +305,7 @@ impl KnownModule {
         }
     }
 
-    pub fn name(self) -> ModuleName {
+    pub(crate) fn name(self) -> ModuleName {
         ModuleName::new_static(self.as_str())
             .unwrap_or_else(|| panic!("{self} should be a valid module name!"))
     }
@@ -301,27 +321,23 @@ impl KnownModule {
         }
     }
 
-    pub const fn is_builtins(self) -> bool {
+    pub(crate) const fn is_builtins(self) -> bool {
         matches!(self, Self::Builtins)
     }
 
-    pub const fn is_typing(self) -> bool {
+    pub(crate) const fn is_typing(self) -> bool {
         matches!(self, Self::Typing)
     }
 
-    pub const fn is_ty_extensions(self) -> bool {
+    pub(crate) const fn is_ty_extensions(self) -> bool {
         matches!(self, Self::TyExtensions)
     }
 
-    pub const fn is_inspect(self) -> bool {
+    pub(crate) const fn is_inspect(self) -> bool {
         matches!(self, Self::Inspect)
     }
 
-    pub const fn is_enum(self) -> bool {
-        matches!(self, Self::Enum)
-    }
-
-    pub const fn is_importlib(self) -> bool {
+    pub(crate) const fn is_importlib(self) -> bool {
         matches!(self, Self::ImportLib)
     }
 }
