@@ -222,7 +222,6 @@ impl<'db> Completion<'db> {
                 // "struct" here as a more general "object." ---AG
                 Type::NominalInstance(_)
                 | Type::PropertyInstance(_)
-                | Type::Tuple(_)
                 | Type::BoundSuper(_)
                 | Type::TypedDict(_) => CompletionKind::Struct,
                 Type::IntLiteral(_)
@@ -244,6 +243,7 @@ impl<'db> Completion<'db> {
                 | Type::KnownInstance(_)
                 | Type::AlwaysTruthy
                 | Type::AlwaysFalsy => return None,
+                Type::TypeAlias(alias) => imp(db, alias.value_type(db))?,
             })
         }
         imp(db, self.ty)
