@@ -1,20 +1,87 @@
 use std::borrow::Cow;
 
+use ruff_db::parsed::ParsedModuleRef;
 use ruff_python_ast as ast;
 use rustc_hash::FxHashMap;
 
 use crate::semantic_index::SemanticIndex;
+use crate::semantic_index::definition::Definition;
+use crate::semantic_index::scope::{FileScopeId, NodeWithScopeKind};
 use crate::types::class::ClassType;
 use crate::types::class_base::ClassBase;
-use crate::types::instance::{NominalInstanceType, Protocol, ProtocolInstanceType};
+use crate::types::infer::infer_definition_types;
+use crate::types::instance::{Protocol, ProtocolInstanceType};
 use crate::types::signatures::{Parameter, Parameters, Signature};
-use crate::types::tuple::{TupleSpec, TupleType};
+use crate::types::tuple::{TupleSpec, TupleType, walk_tuple_type};
 use crate::types::{
-    KnownInstanceType, Type, TypeMapping, TypeRelation, TypeVarBoundOrConstraints, TypeVarInstance,
-    TypeVarVariance, TypeVisitor, UnionType, declaration_type,
+    ApplyTypeMappingVisitor, BoundTypeVarInstance, HasRelationToVisitor, KnownClass,
+    KnownInstanceType, NormalizedVisitor, Type, TypeMapping, TypeRelation, TypeTransformer,
+    TypeVarBoundOrConstraints, TypeVarInstance, TypeVarVariance, UnionType, binding_type,
+    declaration_type,
 };
 use crate::{Db, FxOrderSet};
 
+/// Returns an iterator of any generic context introduced by the given scope or any enclosing
+/// scope.
+fn enclosing_generic_contexts<'db>(
+    db: &'db dyn Db,
+    module: &ParsedModuleRef,
+    index: &SemanticIndex<'db>,
+    scope: FileScopeId,
+) -> impl Iterator<Item = GenericContext<'db>> {
+    index
+        .ancestor_scopes(scope)
+        .filter_map(|(_, ancestor_scope)| match ancestor_scope.node() {
+            NodeWithScopeKind::Class(class) => {
+                let definition = index.expect_single_definition(class.node(module));
+                binding_type(db, definition)
+                    .into_class_literal()?
+                    .generic_context(db)
+            }
+            NodeWithScopeKind::Function(function) => {
+                let definition = index.expect_single_definition(function.node(module));
+                infer_definition_types(db, definition)
+                    .undecorated_type()
+                    .expect("function should have undecorated type")
+                    .into_function_literal()?
+                    .last_definition_signature(db)
+                    .generic_context
+            }
+            _ => None,
+        })
+}
+
+/// Binds an unbound typevar.
+///
+/// When a typevar is first created, we will have a [`TypeVarInstance`] which does not have an
+/// associated binding context. When the typevar is used in a generic class or function, we "bind"
+/// it, adding the [`Definition`] of the generic class or function as its "binding context".
+///
+/// When an expression resolves to a typevar, our inferred type will refer to the unbound
+/// [`TypeVarInstance`] from when the typevar was first created. This function walks the scopes
+/// that enclosing the expression, looking for the innermost binding context that binds the
+/// typevar.
+///
+/// If no enclosing scope has already bound the typevar, we might be in a syntactic position that
+/// is about to bind it (indicated by a non-`None` `typevar_binding_context`), in which case we
+/// bind the typevar with that new binding context.
+pub(crate) fn bind_typevar<'db>(
+    db: &'db dyn Db,
+    module: &ParsedModuleRef,
+    index: &SemanticIndex<'db>,
+    containing_scope: FileScopeId,
+    typevar_binding_context: Option<Definition<'db>>,
+    typevar: TypeVarInstance<'db>,
+) -> Option<BoundTypeVarInstance<'db>> {
+    enclosing_generic_contexts(db, module, index, containing_scope)
+        .find_map(|enclosing_context| enclosing_context.binds_typevar(db, typevar))
+        .or_else(|| {
+            typevar_binding_context.map(|typevar_binding_context| {
+                typevar.with_binding_context(db, typevar_binding_context)
+            })
+        })
+}
+
 /// A list of formal type variables for a generic function, class, or type alias.
 ///
 /// TODO: Handle nested generic contexts better, with actual parent links to the lexically
@@ -23,11 +90,21 @@ use crate::{Db, FxOrderSet};
 /// # Ordering
 /// Ordering is based on the context's salsa-assigned id and not on its values.
 /// The id may change between runs, or when the context was garbage collected and recreated.
-#[salsa::interned(debug)]
+#[salsa::interned(debug, heap_size=GenericContext::heap_size)]
 #[derive(PartialOrd, Ord)]
 pub struct GenericContext<'db> {
     #[returns(ref)]
-    pub(crate) variables: FxOrderSet<TypeVarInstance<'db>>,
+    pub(crate) variables: FxOrderSet<BoundTypeVarInstance<'db>>,
+}
+
+pub(super) fn walk_generic_context<'db, V: super::visitor::TypeVisitor<'db> + ?Sized>(
+    db: &'db dyn Db,
+    context: GenericContext<'db>,
+    visitor: &V,
+) {
+    for bound_typevar in context.variables(db) {
+        visitor.visit_bound_type_var_type(db, *bound_typevar);
+    }
 }
 
 // The Salsa heap is tracked separately.
@@ -38,11 +115,14 @@ impl<'db> GenericContext<'db> {
     pub(crate) fn from_type_params(
         db: &'db dyn Db,
         index: &'db SemanticIndex<'db>,
+        binding_context: Definition<'db>,
         type_params_node: &ast::TypeParams,
     ) -> Self {
         let variables: FxOrderSet<_> = type_params_node
             .iter()
-            .filter_map(|type_param| Self::variable_from_type_param(db, index, type_param))
+            .filter_map(|type_param| {
+                Self::variable_from_type_param(db, index, binding_context, type_param)
+            })
             .collect();
         Self::new(db, variables)
     }
@@ -50,8 +130,9 @@ impl<'db> GenericContext<'db> {
     fn variable_from_type_param(
         db: &'db dyn Db,
         index: &'db SemanticIndex<'db>,
+        binding_context: Definition<'db>,
         type_param_node: &ast::TypeParam,
-    ) -> Option<TypeVarInstance<'db>> {
+    ) -> Option<BoundTypeVarInstance<'db>> {
         match type_param_node {
             ast::TypeParam::TypeVar(node) => {
                 let definition = index.expect_single_definition(node);
@@ -60,7 +141,7 @@ impl<'db> GenericContext<'db> {
                 else {
                     return None;
                 };
-                Some(typevar)
+                Some(typevar.with_binding_context(db, binding_context))
             }
             // TODO: Support these!
             ast::TypeParam::ParamSpec(_) => None,
@@ -72,21 +153,24 @@ impl<'db> GenericContext<'db> {
     /// list.
     pub(crate) fn from_function_params(
         db: &'db dyn Db,
+        definition: Definition<'db>,
         parameters: &Parameters<'db>,
         return_type: Option<Type<'db>>,
     ) -> Option<Self> {
+        // Find all of the legacy typevars mentioned in the function signature.
         let mut variables = FxOrderSet::default();
         for param in parameters {
             if let Some(ty) = param.annotated_type() {
-                ty.find_legacy_typevars(db, &mut variables);
+                ty.find_legacy_typevars(db, Some(definition), &mut variables);
             }
             if let Some(ty) = param.default_type() {
-                ty.find_legacy_typevars(db, &mut variables);
+                ty.find_legacy_typevars(db, Some(definition), &mut variables);
             }
         }
         if let Some(ty) = return_type {
-            ty.find_legacy_typevars(db, &mut variables);
+            ty.find_legacy_typevars(db, Some(definition), &mut variables);
         }
+
         if variables.is_empty() {
             return None;
         }
@@ -101,7 +185,7 @@ impl<'db> GenericContext<'db> {
     ) -> Option<Self> {
         let mut variables = FxOrderSet::default();
         for base in bases {
-            base.find_legacy_typevars(db, &mut variables);
+            base.find_legacy_typevars(db, None, &mut variables);
         }
         if variables.is_empty() {
             return None;
@@ -122,7 +206,11 @@ impl<'db> GenericContext<'db> {
         Signature::new(parameters, None)
     }
 
-    fn parameter_from_typevar(db: &'db dyn Db, typevar: TypeVarInstance<'db>) -> Parameter<'db> {
+    fn parameter_from_typevar(
+        db: &'db dyn Db,
+        bound_typevar: BoundTypeVarInstance<'db>,
+    ) -> Parameter<'db> {
+        let typevar = bound_typevar.typevar(db);
         let mut parameter = Parameter::positional_only(Some(typevar.name(db).clone()));
         match typevar.bound_or_constraints(db) {
             Some(TypeVarBoundOrConstraints::UpperBound(bound)) => {
@@ -137,14 +225,28 @@ impl<'db> GenericContext<'db> {
             }
             None => {}
         }
-        if let Some(default_ty) = typevar.default_ty(db) {
+        if let Some(default_ty) = bound_typevar.default_type(db) {
             parameter = parameter.with_default_type(default_ty);
         }
         parameter
     }
 
-    pub(crate) fn default_specialization(self, db: &'db dyn Db) -> Specialization<'db> {
-        self.specialize_partial(db, &vec![None; self.variables(db).len()])
+    pub(crate) fn default_specialization(
+        self,
+        db: &'db dyn Db,
+        known_class: Option<KnownClass>,
+    ) -> Specialization<'db> {
+        let partial = self.specialize_partial(db, &vec![None; self.variables(db).len()]);
+        if known_class == Some(KnownClass::Tuple) {
+            Specialization::new(
+                db,
+                self,
+                partial.types(db),
+                Some(TupleType::homogeneous(db, Type::unknown())),
+            )
+        } else {
+            partial
+        }
     }
 
     pub(crate) fn identity_specialization(self, db: &'db dyn Db) -> Specialization<'db> {
@@ -161,10 +263,31 @@ impl<'db> GenericContext<'db> {
         self.specialize(db, types.into())
     }
 
+    /// Returns a tuple type of the typevars introduced by this generic context.
+    pub(crate) fn as_tuple(self, db: &'db dyn Db) -> Type<'db> {
+        Type::heterogeneous_tuple(
+            db,
+            self.variables(db)
+                .iter()
+                .map(|typevar| Type::TypeVar(*typevar)),
+        )
+    }
+
     pub(crate) fn is_subset_of(self, db: &'db dyn Db, other: GenericContext<'db>) -> bool {
         self.variables(db).is_subset(other.variables(db))
     }
 
+    pub(crate) fn binds_typevar(
+        self,
+        db: &'db dyn Db,
+        typevar: TypeVarInstance<'db>,
+    ) -> Option<BoundTypeVarInstance<'db>> {
+        self.variables(db)
+            .iter()
+            .find(|self_bound_typevar| self_bound_typevar.typevar(db) == typevar)
+            .copied()
+    }
+
     /// Creates a specialization of this generic context. Panics if the length of `types` does not
     /// match the number of typevars in the generic context. You must provide a specific type for
     /// each typevar; no defaults are used. (Use [`specialize_partial`](Self::specialize_partial)
@@ -182,9 +305,9 @@ impl<'db> GenericContext<'db> {
     pub(crate) fn specialize_tuple(
         self,
         db: &'db dyn Db,
+        element_type: Type<'db>,
         tuple: TupleType<'db>,
     ) -> Specialization<'db> {
-        let element_type = UnionType::from_elements(db, tuple.tuple(db).all_elements());
         Specialization::new(db, self, Box::from([element_type]), Some(tuple))
     }
 
@@ -214,7 +337,7 @@ impl<'db> GenericContext<'db> {
                 continue;
             }
 
-            let Some(default) = typevar.default_ty(db) else {
+            let Some(default) = typevar.default_type(db) else {
                 continue;
             };
 
@@ -233,14 +356,18 @@ impl<'db> GenericContext<'db> {
         Specialization::new(db, self, expanded.into_boxed_slice(), None)
     }
 
-    pub(crate) fn normalized_impl(self, db: &'db dyn Db, visitor: &mut TypeVisitor<'db>) -> Self {
+    pub(crate) fn normalized_impl(self, db: &'db dyn Db, visitor: &NormalizedVisitor<'db>) -> Self {
         let variables: FxOrderSet<_> = self
             .variables(db)
             .iter()
-            .map(|ty| ty.normalized_impl(db, visitor))
+            .map(|bound_typevar| bound_typevar.normalized_impl(db, visitor))
             .collect();
         Self::new(db, variables)
     }
+
+    fn heap_size((variables,): &(FxOrderSet<BoundTypeVarInstance<'db>>,)) -> usize {
+        ruff_memory_usage::order_set_heap_size(variables)
+    }
 }
 
 #[derive(Debug, Copy, Clone, PartialEq, Eq)]
@@ -268,7 +395,7 @@ impl std::fmt::Display for LegacyGenericBase {
 ///
 /// TODO: Handle nested specializations better, with actual parent links to the specialization of
 /// the lexically containing context.
-#[salsa::interned(debug)]
+#[salsa::interned(debug, heap_size=ruff_memory_usage::heap_size)]
 pub struct Specialization<'db> {
     pub(crate) generic_context: GenericContext<'db>,
     #[returns(deref)]
@@ -279,29 +406,40 @@ pub struct Specialization<'db> {
     tuple_inner: Option<TupleType<'db>>,
 }
 
+// The Salsa heap is tracked separately.
+impl get_size2::GetSize for Specialization<'_> {}
+
+pub(super) fn walk_specialization<'db, V: super::visitor::TypeVisitor<'db> + ?Sized>(
+    db: &'db dyn Db,
+    specialization: Specialization<'db>,
+    visitor: &V,
+) {
+    walk_generic_context(db, specialization.generic_context(db), visitor);
+    for ty in specialization.types(db) {
+        visitor.visit_type(db, *ty);
+    }
+    if let Some(tuple) = specialization.tuple_inner(db) {
+        walk_tuple_type(db, tuple, visitor);
+    }
+}
+
 impl<'db> Specialization<'db> {
     /// Returns the tuple spec for a specialization of the `tuple` class.
-    pub(crate) fn tuple(self, db: &'db dyn Db) -> &'db TupleSpec<'db> {
-        if let Some(tuple) = self.tuple_inner(db).map(|tuple_type| tuple_type.tuple(db)) {
-            return tuple;
-        }
-        if let [element_type] = self.types(db) {
-            if let Some(tuple) = TupleType::new(db, TupleSpec::homogeneous(*element_type)) {
-                return tuple.tuple(db);
-            }
-        }
-        TupleType::new(db, TupleSpec::homogeneous(Type::unknown()))
-            .expect("tuple[Unknown, ...] should never contain Never")
-            .tuple(db)
+    pub(crate) fn tuple(self, db: &'db dyn Db) -> Option<&'db TupleSpec<'db>> {
+        self.tuple_inner(db).map(|tuple_type| tuple_type.tuple(db))
     }
 
     /// Returns the type that a typevar is mapped to, or None if the typevar isn't part of this
     /// mapping.
-    pub(crate) fn get(self, db: &'db dyn Db, typevar: TypeVarInstance<'db>) -> Option<Type<'db>> {
+    pub(crate) fn get(
+        self,
+        db: &'db dyn Db,
+        bound_typevar: BoundTypeVarInstance<'db>,
+    ) -> Option<Type<'db>> {
         let index = self
             .generic_context(db)
             .variables(db)
-            .get_index_of(&typevar)?;
+            .get_index_of(&bound_typevar)?;
         self.types(db).get(index).copied()
     }
 
@@ -326,15 +464,24 @@ impl<'db> Specialization<'db> {
         self,
         db: &'db dyn Db,
         type_mapping: &TypeMapping<'a, 'db>,
+    ) -> Self {
+        self.apply_type_mapping_impl(db, type_mapping, &TypeTransformer::default())
+    }
+
+    pub(crate) fn apply_type_mapping_impl<'a>(
+        self,
+        db: &'db dyn Db,
+        type_mapping: &TypeMapping<'a, 'db>,
+        visitor: &ApplyTypeMappingVisitor<'db>,
     ) -> Self {
         let types: Box<[_]> = self
             .types(db)
             .iter()
-            .map(|ty| ty.apply_type_mapping(db, type_mapping))
+            .map(|ty| ty.apply_type_mapping_impl(db, type_mapping, visitor))
             .collect();
         let tuple_inner = self
             .tuple_inner(db)
-            .and_then(|tuple| tuple.apply_type_mapping(db, type_mapping));
+            .and_then(|tuple| tuple.apply_type_mapping_impl(db, type_mapping, visitor));
         Specialization::new(db, self.generic_context(db), types, tuple_inner)
     }
 
@@ -376,7 +523,7 @@ impl<'db> Specialization<'db> {
         Specialization::new(db, self.generic_context(db), types, None)
     }
 
-    pub(crate) fn normalized_impl(self, db: &'db dyn Db, visitor: &mut TypeVisitor<'db>) -> Self {
+    pub(crate) fn normalized_impl(self, db: &'db dyn Db, visitor: &NormalizedVisitor<'db>) -> Self {
         let types: Box<[_]> = self
             .types(db)
             .iter()
@@ -395,8 +542,8 @@ impl<'db> Specialization<'db> {
             .variables(db)
             .into_iter()
             .zip(self.types(db))
-            .map(|(typevar, vartype)| {
-                let variance = match typevar.variance(db) {
+            .map(|(bound_typevar, vartype)| {
+                let variance = match bound_typevar.typevar(db).variance(db) {
                     TypeVarVariance::Invariant => TypeVarVariance::Invariant,
                     TypeVarVariance::Covariant => variance,
                     TypeVarVariance::Contravariant => variance.flip(),
@@ -412,11 +559,12 @@ impl<'db> Specialization<'db> {
         Specialization::new(db, self.generic_context(db), types, tuple_inner)
     }
 
-    pub(crate) fn has_relation_to(
+    pub(crate) fn has_relation_to_impl(
         self,
         db: &'db dyn Db,
         other: Self,
         relation: TypeRelation,
+        visitor: &HasRelationToVisitor<'db>,
     ) -> bool {
         let generic_context = self.generic_context(db);
         if generic_context != other.generic_context(db) {
@@ -425,10 +573,10 @@ impl<'db> Specialization<'db> {
 
         if let (Some(self_tuple), Some(other_tuple)) = (self.tuple_inner(db), other.tuple_inner(db))
         {
-            return self_tuple.has_relation_to(db, other_tuple, relation);
+            return self_tuple.has_relation_to_impl(db, other_tuple, relation, visitor);
         }
 
-        for ((typevar, self_type), other_type) in (generic_context.variables(db).into_iter())
+        for ((bound_typevar, self_type), other_type) in (generic_context.variables(db).into_iter())
             .zip(self.types(db))
             .zip(other.types(db))
         {
@@ -445,7 +593,7 @@ impl<'db> Specialization<'db> {
             //   - contravariant: verify that other_type <: self_type
             //   - invariant: verify that self_type <: other_type AND other_type <: self_type
             //   - bivariant: skip, can't make subtyping/assignability false
-            let compatible = match typevar.variance(db) {
+            let compatible = match bound_typevar.typevar(db).variance(db) {
                 TypeVarVariance::Invariant => match relation {
                     TypeRelation::Subtyping => self_type.is_equivalent_to(db, *other_type),
                     TypeRelation::Assignability => {
@@ -453,9 +601,11 @@ impl<'db> Specialization<'db> {
                             && other_type.is_assignable_to(db, *self_type)
                     }
                 },
-                TypeVarVariance::Covariant => self_type.has_relation_to(db, *other_type, relation),
+                TypeVarVariance::Covariant => {
+                    self_type.has_relation_to_impl(db, *other_type, relation, visitor)
+                }
                 TypeVarVariance::Contravariant => {
-                    other_type.has_relation_to(db, *self_type, relation)
+                    other_type.has_relation_to_impl(db, *self_type, relation, visitor)
                 }
                 TypeVarVariance::Bivariant => true,
             };
@@ -473,7 +623,7 @@ impl<'db> Specialization<'db> {
             return false;
         }
 
-        for ((typevar, self_type), other_type) in (generic_context.variables(db).into_iter())
+        for ((bound_typevar, self_type), other_type) in (generic_context.variables(db).into_iter())
             .zip(self.types(db))
             .zip(other.types(db))
         {
@@ -483,7 +633,7 @@ impl<'db> Specialization<'db> {
             //   - contravariant: verify that other_type == self_type
             //   - invariant: verify that self_type == other_type
             //   - bivariant: skip, can't make equivalence false
-            let compatible = match typevar.variance(db) {
+            let compatible = match bound_typevar.typevar(db).variance(db) {
                 TypeVarVariance::Invariant
                 | TypeVarVariance::Covariant
                 | TypeVarVariance::Contravariant => self_type.is_equivalent_to(db, *other_type),
@@ -494,17 +644,30 @@ impl<'db> Specialization<'db> {
             }
         }
 
+        match (self.tuple_inner(db), other.tuple_inner(db)) {
+            (Some(_), None) | (None, Some(_)) => return false,
+            (None, None) => {}
+            (Some(self_tuple), Some(other_tuple)) => {
+                if !self_tuple.is_equivalent_to(db, other_tuple) {
+                    return false;
+                }
+            }
+        }
+
         true
     }
 
     pub(crate) fn find_legacy_typevars(
         self,
         db: &'db dyn Db,
-        typevars: &mut FxOrderSet<TypeVarInstance<'db>>,
+        binding_context: Option<Definition<'db>>,
+        typevars: &mut FxOrderSet<BoundTypeVarInstance<'db>>,
     ) {
         for ty in self.types(db) {
-            ty.find_legacy_typevars(db, typevars);
+            ty.find_legacy_typevars(db, binding_context, typevars);
         }
+        // A tuple's specialization will include all of its element types, so we don't need to also
+        // look in `self.tuple`.
     }
 }
 
@@ -512,17 +675,35 @@ impl<'db> Specialization<'db> {
 ///
 /// You will usually use [`Specialization`] instead of this type. This type is used when we need to
 /// substitute types for type variables before we have fully constructed a [`Specialization`].
-#[derive(Clone, Debug, Eq, Hash, PartialEq)]
+#[derive(Clone, Debug, Eq, Hash, PartialEq, get_size2::GetSize)]
 pub struct PartialSpecialization<'a, 'db> {
     generic_context: GenericContext<'db>,
     types: Cow<'a, [Type<'db>]>,
 }
 
+pub(super) fn walk_partial_specialization<'db, V: super::visitor::TypeVisitor<'db> + ?Sized>(
+    db: &'db dyn Db,
+    specialization: &PartialSpecialization<'_, 'db>,
+    visitor: &V,
+) {
+    walk_generic_context(db, specialization.generic_context, visitor);
+    for ty in &*specialization.types {
+        visitor.visit_type(db, *ty);
+    }
+}
+
 impl<'db> PartialSpecialization<'_, 'db> {
     /// Returns the type that a typevar is mapped to, or None if the typevar isn't part of this
     /// mapping.
-    pub(crate) fn get(&self, db: &'db dyn Db, typevar: TypeVarInstance<'db>) -> Option<Type<'db>> {
-        let index = self.generic_context.variables(db).get_index_of(&typevar)?;
+    pub(crate) fn get(
+        &self,
+        db: &'db dyn Db,
+        bound_typevar: BoundTypeVarInstance<'db>,
+    ) -> Option<Type<'db>> {
+        let index = self
+            .generic_context
+            .variables(db)
+            .get_index_of(&bound_typevar)?;
         self.types.get(index).copied()
     }
 
@@ -536,7 +717,7 @@ impl<'db> PartialSpecialization<'_, 'db> {
     pub(crate) fn normalized_impl(
         &self,
         db: &'db dyn Db,
-        visitor: &mut TypeVisitor<'db>,
+        visitor: &NormalizedVisitor<'db>,
     ) -> PartialSpecialization<'db, 'db> {
         let generic_context = self.generic_context.normalized_impl(db, visitor);
         let types: Cow<_> = self
@@ -556,7 +737,7 @@ impl<'db> PartialSpecialization<'_, 'db> {
 /// specialization of a generic function.
 pub(crate) struct SpecializationBuilder<'db> {
     db: &'db dyn Db,
-    types: FxHashMap<TypeVarInstance<'db>, Type<'db>>,
+    types: FxHashMap<BoundTypeVarInstance<'db>, Type<'db>>,
 }
 
 impl<'db> SpecializationBuilder<'db> {
@@ -575,16 +756,16 @@ impl<'db> SpecializationBuilder<'db> {
                 self.types
                     .get(variable)
                     .copied()
-                    .unwrap_or(variable.default_ty(self.db).unwrap_or(Type::unknown()))
+                    .unwrap_or(variable.default_type(self.db).unwrap_or(Type::unknown()))
             })
             .collect();
         // TODO Infer the tuple spec for a tuple type
         Specialization::new(self.db, generic_context, types, None)
     }
 
-    fn add_type_mapping(&mut self, typevar: TypeVarInstance<'db>, ty: Type<'db>) {
+    fn add_type_mapping(&mut self, bound_typevar: BoundTypeVarInstance<'db>, ty: Type<'db>) {
         self.types
-            .entry(typevar)
+            .entry(bound_typevar)
             .and_modify(|existing| {
                 *existing = UnionType::from_elements(self.db, [*existing, ty]);
             })
@@ -620,89 +801,6 @@ impl<'db> SpecializationBuilder<'db> {
         }
 
         match (formal, actual) {
-            (Type::TypeVar(typevar), ty) | (ty, Type::TypeVar(typevar)) => {
-                match typevar.bound_or_constraints(self.db) {
-                    Some(TypeVarBoundOrConstraints::UpperBound(bound)) => {
-                        if !ty.is_assignable_to(self.db, bound) {
-                            return Err(SpecializationError::MismatchedBound {
-                                typevar,
-                                argument: ty,
-                            });
-                        }
-                        self.add_type_mapping(typevar, ty);
-                    }
-                    Some(TypeVarBoundOrConstraints::Constraints(constraints)) => {
-                        for constraint in constraints.iter(self.db) {
-                            if ty.is_assignable_to(self.db, *constraint) {
-                                self.add_type_mapping(typevar, *constraint);
-                                return Ok(());
-                            }
-                        }
-                        return Err(SpecializationError::MismatchedConstraint {
-                            typevar,
-                            argument: ty,
-                        });
-                    }
-                    _ => {
-                        self.add_type_mapping(typevar, ty);
-                    }
-                }
-            }
-
-            (Type::Tuple(formal_tuple), Type::Tuple(actual_tuple)) => {
-                let formal_tuple = formal_tuple.tuple(self.db);
-                let actual_tuple = actual_tuple.tuple(self.db);
-                match (formal_tuple, actual_tuple) {
-                    (TupleSpec::Fixed(formal_tuple), TupleSpec::Fixed(actual_tuple)) => {
-                        if formal_tuple.len() == actual_tuple.len() {
-                            for (formal_element, actual_element) in formal_tuple.elements().zip(actual_tuple.elements()) {
-                                self.infer(*formal_element, *actual_element)?;
-                            }
-                        }
-                    }
-
-                    // TODO: Infer specializations of variable-length tuples
-                    (TupleSpec::Variable(_), _) | (_, TupleSpec::Variable(_)) => {}
-                }
-            }
-
-            (
-                Type::NominalInstance(NominalInstanceType {
-                    class: ClassType::Generic(formal_alias),
-                    ..
-                })
-                // TODO: This will only handle classes that explicit implement a generic protocol
-                // by listing it as a base class. To handle classes that implicitly implement a
-                // generic protocol, we will need to check the types of the protocol members to be
-                // able to infer the specialization of the protocol that the class implements.
-                | Type::ProtocolInstance(ProtocolInstanceType {
-                    inner: Protocol::FromClass(ClassType::Generic(formal_alias)),
-                    ..
-                }),
-                Type::NominalInstance(NominalInstanceType {
-                    class: actual_class,
-                    ..
-                }),
-            ) => {
-                let formal_origin = formal_alias.origin(self.db);
-                for base in actual_class.iter_mro(self.db) {
-                    let ClassBase::Class(ClassType::Generic(base_alias)) = base else {
-                        continue;
-                    };
-                    if formal_origin != base_alias.origin(self.db) {
-                        continue;
-                    }
-                    let formal_specialization = formal_alias.specialization(self.db).types(self.db);
-                    let base_specialization = base_alias.specialization(self.db).types(self.db);
-                    for (formal_ty, base_ty) in
-                        formal_specialization.iter().zip(base_specialization)
-                    {
-                        self.infer(*formal_ty, *base_ty)?;
-                    }
-                    return Ok(());
-                }
-            }
-
             (Type::Union(formal), _) => {
                 // TODO: We haven't implemented a full unification solver yet. If typevars appear
                 // in multiple union elements, we ideally want to express that _only one_ of them
@@ -714,14 +812,14 @@ impl<'db> SpecializationBuilder<'db> {
                 // and add a mapping between that typevar and the actual type. (Note that we've
                 // already handled above the case where the actual is assignable to a _non-typevar_
                 // union element.)
-                let mut typevars = formal.iter(self.db).filter_map(|ty| match ty {
-                    Type::TypeVar(typevar) => Some(*typevar),
+                let mut bound_typevars = formal.iter(self.db).filter_map(|ty| match ty {
+                    Type::TypeVar(bound_typevar) => Some(*bound_typevar),
                     _ => None,
                 });
-                let typevar = typevars.next();
-                let additional_typevars = typevars.next();
-                if let (Some(typevar), None) = (typevar, additional_typevars) {
-                    self.add_type_mapping(typevar, actual);
+                let bound_typevar = bound_typevars.next();
+                let additional_bound_typevars = bound_typevars.next();
+                if let (Some(bound_typevar), None) = (bound_typevar, additional_bound_typevars) {
+                    self.add_type_mapping(bound_typevar, actual);
                 }
             }
 
@@ -735,6 +833,98 @@ impl<'db> SpecializationBuilder<'db> {
                 }
             }
 
+            (Type::TypeVar(bound_typevar), ty) | (ty, Type::TypeVar(bound_typevar)) => {
+                match bound_typevar.typevar(self.db).bound_or_constraints(self.db) {
+                    Some(TypeVarBoundOrConstraints::UpperBound(bound)) => {
+                        if !ty.is_assignable_to(self.db, bound) {
+                            return Err(SpecializationError::MismatchedBound {
+                                bound_typevar,
+                                argument: ty,
+                            });
+                        }
+                        self.add_type_mapping(bound_typevar, ty);
+                    }
+                    Some(TypeVarBoundOrConstraints::Constraints(constraints)) => {
+                        for constraint in constraints.iter(self.db) {
+                            if ty.is_assignable_to(self.db, *constraint) {
+                                self.add_type_mapping(bound_typevar, *constraint);
+                                return Ok(());
+                            }
+                        }
+                        return Err(SpecializationError::MismatchedConstraint {
+                            bound_typevar,
+                            argument: ty,
+                        });
+                    }
+                    _ => {
+                        self.add_type_mapping(bound_typevar, ty);
+                    }
+                }
+            }
+
+            (formal, Type::NominalInstance(actual_nominal)) => {
+                // Special case: `formal` and `actual` are both tuples.
+                if let (Some(formal_tuple), Some(actual_tuple)) = (
+                    formal.tuple_instance_spec(self.db),
+                    actual_nominal.tuple_spec(self.db),
+                ) {
+                    let Some(most_precise_length) =
+                        formal_tuple.len().most_precise(actual_tuple.len())
+                    else {
+                        return Ok(());
+                    };
+                    let Ok(formal_tuple) = formal_tuple.resize(self.db, most_precise_length) else {
+                        return Ok(());
+                    };
+                    let Ok(actual_tuple) = actual_tuple.resize(self.db, most_precise_length) else {
+                        return Ok(());
+                    };
+                    for (formal_element, actual_element) in
+                        formal_tuple.all_elements().zip(actual_tuple.all_elements())
+                    {
+                        self.infer(*formal_element, *actual_element)?;
+                    }
+                    return Ok(());
+                }
+
+                // Extract formal_alias if this is a generic class
+                let formal_alias = match formal {
+                    Type::NominalInstance(formal_nominal) => {
+                        formal_nominal.class(self.db).into_generic_alias()
+                    }
+                    // TODO: This will only handle classes that explicit implement a generic protocol
+                    // by listing it as a base class. To handle classes that implicitly implement a
+                    // generic protocol, we will need to check the types of the protocol members to be
+                    // able to infer the specialization of the protocol that the class implements.
+                    Type::ProtocolInstance(ProtocolInstanceType {
+                        inner: Protocol::FromClass(ClassType::Generic(alias)),
+                        ..
+                    }) => Some(alias),
+                    _ => None,
+                };
+
+                if let Some(formal_alias) = formal_alias {
+                    let formal_origin = formal_alias.origin(self.db);
+                    for base in actual_nominal.class(self.db).iter_mro(self.db) {
+                        let ClassBase::Class(ClassType::Generic(base_alias)) = base else {
+                            continue;
+                        };
+                        if formal_origin != base_alias.origin(self.db) {
+                            continue;
+                        }
+                        let formal_specialization =
+                            formal_alias.specialization(self.db).types(self.db);
+                        let base_specialization = base_alias.specialization(self.db).types(self.db);
+                        for (formal_ty, base_ty) in
+                            formal_specialization.iter().zip(base_specialization)
+                        {
+                            self.infer(*formal_ty, *base_ty)?;
+                        }
+                        return Ok(());
+                    }
+                }
+            }
+
             // TODO: Add more forms that we can structurally induct into: type[C], callables
             _ => {}
         }
@@ -746,20 +936,20 @@ impl<'db> SpecializationBuilder<'db> {
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub(crate) enum SpecializationError<'db> {
     MismatchedBound {
-        typevar: TypeVarInstance<'db>,
+        bound_typevar: BoundTypeVarInstance<'db>,
         argument: Type<'db>,
     },
     MismatchedConstraint {
-        typevar: TypeVarInstance<'db>,
+        bound_typevar: BoundTypeVarInstance<'db>,
         argument: Type<'db>,
     },
 }
 
 impl<'db> SpecializationError<'db> {
-    pub(crate) fn typevar(&self) -> TypeVarInstance<'db> {
+    pub(crate) fn bound_typevar(&self) -> BoundTypeVarInstance<'db> {
         match self {
-            Self::MismatchedBound { typevar, .. } => *typevar,
-            Self::MismatchedConstraint { typevar, .. } => *typevar,
+            Self::MismatchedBound { bound_typevar, .. } => *bound_typevar,
+            Self::MismatchedConstraint { bound_typevar, .. } => *bound_typevar,
         }
     }
 
