@@ -98,7 +98,10 @@ impl<S> tracing_subscriber::layer::Filter<S> for LogLevelFilter {
         meta: &tracing::Metadata<'_>,
         _: &tracing_subscriber::layer::Context<'_, S>,
     ) -> bool {
-        let filter = if meta.target().starts_with("ty") || meta.target().starts_with("ruff") {
+        let filter = if meta.target().starts_with("ty")
+            || meta.target().starts_with("ruff")
+            || meta.target().starts_with("e2e")
+        {
             self.filter.trace_level()
         } else {
             tracing::Level::WARN
